import math
import pyglet

import pymunk as pm
from pymunk.vec2d import Vec2d

import data


# why can't I use this BaseSprite directly - doesn't render??
class BaseSprite(pyglet.sprite.Sprite):
	dx = 0
	dy = 0
	drotation = 0
	
	is_destroyed = False
	
	def __init__(self, img, batch=None):
		super(BaseSprite, self).__init__(img, 100, 100, batch=batch)
		if (isinstance(img, pyglet.image.AbstractImage)):
			data.centre_image(img)
		self.reset()

	def reset(self):
		self.x = 0
		self.y = 0
		self.rotation = 0

class CollideSprite(BaseSprite):
	collide_type = "sprite"
	
	body = None
	shape = None
	mass = 1.
	radius = 10.
	friction = 5.
	
	shape_group = None
	in_world = False

	def __init__(self, img, batch=None):
		super(CollideSprite, self).__init__(img, batch=batch)
		self.init_physics()
	
	def moveto(self, x, y):
		self.x, self.y = x,y
		self.body.position = self.x, self.y
	
	def update(self, dt):
		active_force = self.dx *1000., self.dy * 1000.
		friction_force = self.body.velocity * -(self.friction * self.mass)
		resultant_force = active_force + friction_force
		
		offset = self.x,self.y
		self.body.reset_forces()
		self.body.apply_force(resultant_force, offset)
		#self.body.apply_impulse(resultant_force, offset)
		
		self.x = int(self.body.position.x)
		self.y = int(self.body.position.y)

		self.dx,self.dy = 0,0
		
		#super(CollideSprite, self).update(dt)
		
	def physics_shape(self):
		return pm.Circle(self.body, self.radius, Vec2d(0,0))
		
	def physics_inertia(self):
		return pm.moment_for_circle(self.mass, 0, self.radius, Vec2d(0,0))
		
	def init_physics(self):
		inertia = self.physics_inertia()
		self.body = pm.Body(self.mass, inertia)
		self.shape = self.physics_shape()
		
		self.body.position = self.x, self.y
		if ( self.shape_group != None ):
			self.shape.group = self.shape_group
		
		self.shape.data = self
		
		#data.space.add(self.body, self.shape)
	
	def die(self):
		self.is_destroyed = True
		
	def remove(self):
		if (self.in_world):
			data.space.remove(self.shape, self.body)
			self.in_world = False
			
	def store(self):
		self.remove()
		
	def restore(self):
		data.space.add(self.body, self.shape)
		self.in_world = True

class Character(CollideSprite):
	collide_type = "character"
	
	move_speed = 500.
	rotate_speed = 30.
	
	health = 100.
	
	def __init__(self, img, batch=None):
		super(Character, self).__init__(img, batch=batch)
		
	def update(self, dt):
		super(Character, self).update(dt)

	def hurt(self, amount):
		self.health -= amount
		if ( self.health <= 0 ):
			self.die()
		
	def movetowards(self, x, y, speed=None, rotate=True):
		# move towards the target location
		if ( speed == None ):
			speed = self.move_speed / 50
			
		xdelta = x - self.x
		ydelta = y - self.y
		
		targetdist = data.dist(x, y, self.x, self.y)
		if ( targetdist > 0 ):
			percent = (speed / targetdist)
			if ( percent > 1 ):
				percent = 1
			self.dx += percent * xdelta
			self.dy += percent * ydelta
			
			if ( rotate ):
				#automatically face our target
				self.rotation = data.vector_to_rotation( xdelta, ydelta )
		
	def rotatetowards(self, x, y, speed=None, move=True):
		# rotate towards the target location, at our max speed
		if ( speed == None ):
			speed = self.rotate_speed / 50
			
		xdelta = x - self.x
		ydelta = y - self.y
		
		targetrot = data.vector_to_rotation( xdelta, ydelta )
		rotdelta = data.clip(targetrot - self.rotation, -180, 180)
		
		rotamount = rotdelta
		if ( rotdelta > 0 ):
			rotamount = min([rotdelta, speed])
		if ( rotdelta < 0 ):
			rotamount = max([rotdelta, -speed])
		
		self.rotation += rotamount
		
		if ( move ):
			#move speed depends on how close we are to the right direction - close = fast
			move_speed = (1.0 - math.fabs(rotdelta) / 180.) * (self.move_speed / 50)
			move_vector = data.rotation_to_vector( self.rotation, move_speed )
			self.dx += move_vector[0]
			self.dy += move_vector[1]
			
			#print "updating self dx,dy by " + str(move_vector)

